#!/usr/bin/env python
import os, sys, threading, Queue, getopt
from Tkinter import *
from ScrolledText import ScrolledText
from nws.client import NetWorkSpace, FIFO

def sendMsg():
    msg = ent.get().strip()
    if msg:
        ws.store(varName, "%s: %s" % (user, msg))
    ent.delete(0, END)

def checkForNewMessages():
    txt.config(state=NORMAL)
    for x in proxy.findAll():
        y = x.split(':', 1)
        if len(y) == 2:
            user = y[0].strip().lower()
            msg = y[1].strip()
            if user and msg:
                p = config.picture(user)
                if p:
                    txt.image_create(END, image=p)
                else:
                    txt.insert(END, user + ':')

                txt.insert(END, ' ')
                tag = config.style(user, txt)
                if tag:
                    txt.insert(END, msg, tag)
                else:
                    txt.insert(END, msg)
                txt.insert(END, '\n\n')
                txt.see(END)
    txt.config(state=DISABLED)
    txt.after(1000, checkForNewMessages)

class Config:
    def __init__(self, configFile, imageDir):
        self._configFile = configFile
        try:
            self._config = dict(eval(open(configFile).read()))
        except:
            self._config = {}

        if imageDir:
            self._imageDir = imageDir
        else:
            self._imageDir = self._config.get('imageDir', '.')

        self._pictures = {}
        self._styles = {}

    def style(self, user, txt):
        s = None
        try:
            styleName = self._config[user]['style']
            if not self._styles.has_key(styleName):
                try:
                    txt.tag_config(styleName, **self._config['styles'][styleName])
                    s = self._styles[styleName] = styleName  # just a marker
                except KeyError:
                    # error: style defined for user, but no style description
                    print >> sys.stderr, \
                            'error in configuration file %s' % self._configFile
                    print >> sys.stderr, \
                            '    style %s specified for user %s is not defined' % \
                            (styleName, user)
            else:
                # we've already configured this style
                s = styleName
        except KeyError:
            # no error: no style defined for this user
            pass
        return s

    def picture(self, user):
        try:
            pict = self._config[user]['picture']
        except KeyError:
            pict = user

        try:
            p = self._pictures[pict]
        except KeyError:
            try:
                t = os.path.join(self._imageDir, pict + '.gif')
                p = self._pictures[pict] = PhotoImage(file=t)
            except:
                p = self._pictures[pict] = None
        return p

class WorkSpaceProxy(threading.Thread):
    def __init__(self, varName, wsName, **opt):
        threading.Thread.__init__(self, name='WorkSpaceProxy')
        ws = NetWorkSpace(wsName, **opt)
        self._ifind = ws.ifind(varName)
        self._queue = Queue.Queue()

    def run(self):
        for val in self._ifind:
            self._queue.put(val)

    def findAll(self):
        values = []
        while not self._queue.empty():
            values.append(self._queue.get_nowait())
        return values

def usage():
    print >> sys.stderr, "usage: %s [option ...]" % os.path.basename(sys.argv[0])
    print >> sys.stderr, "where option can be:"
    print >> sys.stderr, "  -h <host>"
    print >> sys.stderr, "  -p <port>"
    print >> sys.stderr, "  -n <name of chat room>"
    print >> sys.stderr, "  -d <image directory>"
    print >> sys.stderr, "  -c <configuration file>"
    print >> sys.stderr, "  -u <username>"
    sys.exit(1)

def leaving():
    try: ws.store(varName, "%s: [has exited the chatroom]" % user)
    except: pass

if __name__ == '__main__':
    wsName, varName = 'chatroom', 'chat'
    configFile = os.path.expanduser(os.path.join('~', '.chatrc'))
    imageDir = None
    user = os.environ.get('USER') or os.environ.get('USERNAME' , 'anonymous')
    nwsargs = {}
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'h:p:n:d:c:u:')
        for opt, arg in opts:
            if opt == '-h':
                nwsargs['serverHost'] = arg
            elif opt == '-p':
                nwsargs['serverPort'] = int(arg)
            elif opt == '-n':
                wsName = arg
            elif opt == '-d':
                imageDir = arg
            elif opt == '-c':
                configFile = arg
            elif opt == '-u':
                user = arg
            else:
                raise 'internal error: out-of-sync with getopt'

        if args:
            print >> sys.stderr, "illegal argument(s) specified:", " ".join(args)
            usage()

        ws = NetWorkSpace(wsName, persistent=True, **nwsargs)
        ws.declare(varName, FIFO)
        proxy = WorkSpaceProxy(varName, wsName, **nwsargs)
        proxy.setDaemon(True)
        proxy.start()

        config = Config(configFile, imageDir)

        root = Tk()
        root.title('NWS Chat Client')
        Label(root, text='NWS Chat Client').pack(padx=4, pady=4)
        panel = Frame(root)
        panel.pack(side=BOTTOM, fill=X)
        txt = ScrolledText(root)
        txt.pack(expand=YES, fill=BOTH, padx=4, pady=4)
        Button(panel, text='Exit', command=root.quit).pack(side=LEFT, padx=4, pady=4)
        ent = Entry(panel)
        ent.pack(side=LEFT, expand=YES, fill=X, padx=4, pady=4)
        ent.focus()
        ent.bind('<Return>', lambda event: sendMsg())
        ws.store(varName, "%s: [has entered the chatroom]" % user)
        checkForNewMessages()
        root.mainloop()
        leaving()
    except getopt.GetoptError, e:
        print >> sys.stderr, e.msg
        usage()
    except ValueError, e:
        print >> sys.stderr, "option %s requires an integer argument" % opt
        usage()
    except SystemExit, e:
        leaving()
        sys.exit(e)
    except KeyboardInterrupt:
        leaving()
    except Exception, e:
        print >> sys.stderr, str(e)
