#!/usr/bin/env python

import sys, getopt, cgi, traceback
from nws.client import NetWorkSpace, NwsServerException, NwsConnectException, NwsNoWorkSpaceException

styleBlock = '''\
<style>
  .error {
    color: red;
    margin: 0;
  }
  .info {
    color: blue;
    margin: 1em 0em 1em 0em;
  }
  body {
    border: 0;
    padding: 0;
    margin: 0;
    background-color: #efefef;
  }
  h4 {
    padding: 0.1em;
    background-color: #777;
    color: white;
    border-bottom: thin white dashed;
  }
  h5 {
    padding: 0.1em;
    background-color: #999;
    color: white;
    border-bottom: thin green dashed;
    border-top: thin green dashed;
  }
  input {
    display: block;
    width: 375px;
    float: left;
    margin-bottom: 10px;
  }
  label {
    display: block;
    text-align: right;
    float: left;
    width: 75px;
    padding-right: 20px;
  }
  br {
    clear: left;
  }
  .buttonSubmit {
    width: 75px;
    margin-left: 95px;
  }
  .hidden {
    display: none;
  }
</style>
'''

def htmlReply(monName, wsName, vardict, msg=''):
    reply = '''\
<html>
<head>
<title>Nws Configurator</title>
%s
</head>
<body>
<h4>Nws Configurator</h4>
''' % (styleBlock,)

    if msg:
        reply += '''\
<div class="info"> %s </div>
''' % (msg,)

    if len(vardict) > 0:
        reply += '''\
<form action="doit" method="post">
'''

        for k, v in vardict.items():
            reply += '''\
  <label for="value">%s</label>
  <input type="text" name="value" id="value" value="%s">
  <input type="hidden" name="varName" id="varName" value="%s">
  <br>
''' % (k[len('config'):], v, k)

        reply += '''\
  <input type="hidden" name="op" id="op" value="showMonitor" class="hidden">
  <input type="hidden" name="monName" id="monName" value="%s" class="hidden">
  <input type="hidden" name="wsName" id="wsName" value="%s" class="hidden">
  <input type="submit" value="update" class="buttonSubmit"><br>
</form>
''' % (cgi.escape(monName), cgi.escape(wsName),)

    else:
        reply += '''\
<div class="error"> There are no <em>config</em> variables in this workspace </div>
<div class="info"> Note that <em>config</em> variables begin with the letters "config",
and must have the mode "single" </div>
'''

    reply += '''\
<p>
<h5><a href="doit?op=listVars&wsName=%s"> List variables in %s </a></h5>
<p>
<h5><a href="doit?op=listWss"> List netWorkSpaces </a></h5>
</body>
</html>
''' % (cgi.escape(wsName), cgi.escape(wsName))

    return reply

def htmlError(msg, e):
    return '''\
<html>
<head>
<title>Error</title>
%s
</head>
<body>
<h4>Error</h4>
<div class="error"> %s: %s </div>
<p>
<h5><a href="doit?op=listWss"> List netWorkSpaces </a></h5>
</body>
</html>
''' % (styleBlock, cgi.escape(msg), cgi.escape(str(e)))

class Args:
    def __init__(self, **kw):
        for k, v in kw.items():
            setattr(self, k, v)

def configurator(host, port):
    monName = 'Nws Configurator'
    mon = NetWorkSpace(monName, host, port)

    while 1:
        args = Args(wsName=None, varName=[], value=[], replyVarName='reply')

        try:
            n = int(mon.fetch('request'))
            # print >> sys.stderr, 'n =', n

            for i in range(n):
                a = mon.fetch('request')
                # print >> sys.stderr, 'request =', a
                k, v = a.split('=', 1)
                if hasattr(args, k):
                    x = getattr(args, k)
                    if type(x) == list:
                        x.append(v)
                    else:
                        setattr(args, k, v)
                else:
                    print >> sys.stderr, 'error: received unknown argument ' + k

            ws = mon.server.useWs(args.wsName, create=False)

            # store values into config variables, if any specified
            for k, v in zip(args.varName, args.value):
                # print >> sys.stderr, 'preparing to store:', k, v
                if k.startswith('config') and len(k) > len('config'):
                    ws.store(k, v)
                else:
                    print >> sys.stderr, "variable name didn't use 'config' prefix"

            # get the current values of config variables
            varinfo = ws.listVars().split('\n')
            vars = [v.split('\t') for v in varinfo]
            vardict = {}
            for name, values, fetchers, finders, mode in vars:
                # print >> sys.stderr, name, values, fetchers, finders, mode
                if name.startswith('config') and len(name) > len('config') and mode == 'single':
                    try:
                        v = ws.findTry(name, '')
                        if type(v) == str:
                            # print >> sys.stderr, "setting %s to %s" % (name, v)
                            vardict[name] = v
                        else:
                            print >> sys.stderr, "it wasn't a string"
                    except:
                        pass

            if len(args.varName) > 0: msg = 'Updated workspace'
            else: msg = ''
            t = htmlReply(monName, args.wsName, vardict, msg)
        except ImportError, e:
            t = htmlError('Import error', e)
        except NwsNoWorkSpaceException, e:
            t = htmlError('Error opening workspace', e)
        except ValueError:
            traceback.print_exc()
            continue

        mon.store(args.replyVarName, '0')
        mon.store(args.replyVarName, t)

def main(argv):
    host = 'localhost'
    port = 8765

    try:
        opts, args = getopt.getopt(argv, 'h:p:')

        for opt, arg in opts:
            if opt == '-h':
                host = arg
            elif opt == '-p':
                port = int(arg)
            else:
                raise 'internal error: out-of-sync with getopt'

        if len(args) > 0:
            print >> sys.stderr, 'ignoring unused arguments:', args

        configurator(host, port)
    except getopt.GetoptError, e:
        print >> sys.stderr, e.msg
        sys.exit(1)
    except ValueError, e:
        print >> sys.stderr, 'option %s requires an integer argument' % opt
        sys.exit(1)
    except NwsConnectException, e:
        # the server isn't running
        print >> sys.stderr, e.args[0]
    except (KeyboardInterrupt, NwsServerException, SystemExit):
        # either the user or the server is telling us to shutdown
        pass
    except:
        traceback.print_exc()

if __name__ == '__main__':
    main(sys.argv[1:])
