#!/usr/bin/env python

import unittest, time
from nws.client import SINGLE
from nws.sleigh import Sleigh, sshcmd
from random import randint

try:
    from logging import getLogger, basicConfig, DEBUG
except ImportError:
    DEBUG = 1
    class Logger:
        def __init__(self):
            self._level = 0

        def info(self, msg):
            if self._level:
                print msg

        def setLevel(self, level):
            self._level = level

    def getLogger(ign):
        return Logger()

    def basicConfig():
        pass

def broadcastWorker(n):
    for i in xrange(n):
        SleighNws.store('broadcast', randint(1, 1000))

    workers = SleighNws.fetch('workers') - 1
    SleighNws.store('workers', workers)
    if workers == 0:
        SleighNws.store('broadcast', 0)

    total = 0
    for t in SleighNws.ifind('broadcast'):
        if t == 0: break
        total += t

    return total

def monitorWorker():
    return list(SleighNws.ifind('monitor'))

class SleighTest(unittest.TestCase):
    log = getLogger('SleighTest')
    wsname = 'unit test ws'

    def setUp(self):
        self.log.info('Setting up')
        self.s = Sleigh(**slargs)

    def tearDown(self):
        self.log.info('Tearing down')
        try: self.s.stop()
        except: pass

    def testBroadcast(self):
        """Test all-to-all broadcast using ifind"""
        broadcastWorker.__module__ = 'test_sleigh2'
        self.s.nws.store('workers', self.s.workerCount)
        n = 1000
        results = self.s.eachWorker(broadcastWorker, n)
        self.assertEquals(len(results), self.s.workerCount)

        # Check the results by calling ifindTry
        values = list(self.s.nws.ifindTry('broadcast'))
        self.assertEquals(len(values), n * self.s.workerCount + 1)
        self.assertEquals(values[-1], 0)
        total = sum(values)
        self.assertEquals(results, [total] * len(results))

        # Now let's verify the results using a standard fetchTry loop
        check = 0
        while True:
            t = self.s.nws.fetchTry('broadcast')
            if t is None: break
            check += t

        self.assertEquals(total, check)

    def testMonitor(self):
        """Test one-to-all broadcast with a single mode variable"""
        monitorWorker.__module__ = 'test_sleigh2'

        for n, s in [(1000, 0.0), (100, 0.1), (10, 1.0)]:
            pending = self.s.eachWorker(monitorWorker, blocking=False)
            time.sleep(3) # not necessary for test to pass
            self.s.nws.declare('monitor', SINGLE)
            for i in xrange(n):
                self.s.nws.store('monitor', i)
                time.sleep(s) # not necessary for test to pass
            time.sleep(3) # not necessary for test to pass
            self.s.nws.deleteVar('monitor')
            results = pending.wait()
            self.assertEquals(len(results), self.s.workerCount)
            for r in results:
                self.assert_(len(r) <= n)
                if len(r) > 0:
                    p = r.pop(0)
                    for t in r:
                        self.assert_(t > p)
                        p = t
                else:
                    print 'WARNING: got zero length result'

if __name__ == '__main__':
    import sys, os

    slargs = {}
    try: slargs['nwsHost'] = os.environ['NWS_HOST']
    except: pass
    try: slargs['nwsPort'] = int(os.environ['NWS_PORT'])
    except: pass
    slargs['verbose'] = verbose = os.environ.get('NWS_VERBOSE', False)

    try:
        slargs['nodeList'] = os.environ['NWS_NODES'].split()
        slargs['launch'] = sshcmd
        print "using ssh to launch workers on machines:", slargs['nodeList']
    except:
        print "using default local workers"

    basicConfig()
    log = getLogger('SleighTest')
    if verbose:
        log.setLevel(DEBUG)

    unittest.main()
